//---------------------------------------------------------------------------
//  Program : Smart Filtering
//  Designer: Rich Wang.
//  Date    : April 15, 2004.
//
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "SetFilter0.h"
#include <math.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TSetFilter *SetFilter;

//---------------------------------------------------------------------------
__fastcall TSetFilter::TSetFilter(TComponent* Owner)
        : TForm(Owner)
{
}
// ***** get most closer value in binary format
unsigned char getMinimum(unsigned char gray, unsigned char bitn)
{ unsigned char lt, rt, v[2];

  lt=(0x01<<(bitn+1));
  rt=0xFF>>(8-bitn);
  v[1]= (lt | gray);
  v[1]= v[1] & (~rt);
  v[0]= gray | rt;
  v[0]= v[0] & (~lt);
  if ( abs(gray-v[0])>abs(gray-v[1]) ) return (v[1]);
  else return (v[0]);
}
//---------------------------------------------------------------------------
#define Deg2Rad  3.141592653/180.

// Don't name "Left" because some components use that !!!
enum TextPosition { LEFTUP=1,   UP,     RIGHTUP,
                    LEFT,       CENTER, RIGHT,
                    LEFTBOTTOM, BOTTOM, RIGHTBOTTOM,
                    USERDEFINE };

void putText(TCanvas *canvas, int x, int y, char *text,
             TextPosition tp=LEFTUP, int xc=0, int yc=0)
{ int h, w;
  TBrushStyle oldBrushStyle;
  h=canvas->TextHeight(text);
  w=canvas->TextWidth(text);
  switch(tp)
    { case      LEFTUP : x-=w;      y-=h;      break;
      case          UP : x-=(w>>1); y-=h;      break;
      case     RIGHTUP :            y-=h;      break;
      case        LEFT : x-=w;      y-=(h>>1); break;
      case      CENTER : x-=(w>>1); y-=(h>>1); break;
      case       RIGHT :            y-=(h>>1); break;
      case  LEFTBOTTOM : x-=w;                 break;
      case      BOTTOM : x-=(w>>1);            break;
      case RIGHTBOTTOM : break;
      case  USERDEFINE : x-=xc;     y-=yc;
    };
  oldBrushStyle=canvas->Brush->Style;
  canvas->Brush->Style=bsClear;  //
  canvas->TextOut(x, y, text);
  canvas->Brush->Style=oldBrushStyle;
}

//---------------------------------------------------------------------------

int gSide=3;
float gFilter[49];
TEdit **pFilter=NULL;
//int x0=20, y0=20;
int sx=20, sy=20, width=40, height=25;

void releaseFilter(int side)
{ int index;

  index=side*side-1;
  if ( pFilter!=NULL )
    { while( index>=0 )
        { delete pFilter[index--];
        }
      delete [] pFilter;
      pFilter=NULL;
    }
}
void saveFilter(float *filter, int side)
{ int index=0;
  if ( pFilter!=NULL )
    { side*=side;
      while( index<side )
        { if ( pFilter[index]->Text!="" )
            filter[index]=StrToFloat( pFilter[index]->Text );
          else filter[index]=0;

          ++index;
        }
    }
}
void loadFilter(float *filter, int side)
{ int index=0;

  if ( pFilter!=NULL )
    { side*=side;
      while( index<side )
        { pFilter[index]->Text=FloatToStr( filter[index] );
          ++index;
        }
    }
}
void assignFilter(float value, int side=3)
{ int index=0;

  if ( pFilter!=NULL )
    { side*=side;
      while( index<side )
        { pFilter[index]->Text=FloatToStr( value );
          ++index;
        }
    }
}
void assignFilter(TComponent *parent, int side, float *filter=NULL)
{ int r, c, index, cntr;

  if ( pFilter!=NULL ) releaseFilter(gSide);
  gSide = side;
  cntr= side>>1;
  pFilter= new TEdit*[side*side];
  index=0;
  for (r=0; r<side; ++r)
     { for (c=0; c<side; ++c)
          { pFilter[index]= new TEdit(parent);

            pFilter[index]->Visible= false; // to avoid flicking

            pFilter[index]->Parent = (TWinControl *)parent;
            pFilter[index]->Left   = sx+c*(width+10);
            pFilter[index]->Top    = sy+r*(height+8) ;
            pFilter[index]->Width  = width;
            pFilter[index]->Height = height;
            if ( filter==NULL )
              pFilter[index]->Text="0";
            else pFilter[index]->Text=FloatToStr(gFilter[index]);
            pFilter[index]->Visible= true;
            if ( r==cntr || c==cntr )
              pFilter[index]->Font->Color=clRed;
            else pFilter[index]->Font->Color=clBlack;
            ++index;
          }
     }
}

//===================================
void showXY(TCanvas *canvas, int x0, int y0)
{ int range, r, c, dx=20, dy=20;

  range = (gSide+1) / 2;

  putText(canvas, x0+(range-1)*(width+10)+width/2, y0, "0", UP );
  putText(canvas, x0-5, y0+(range-1)*(height+8)+height/2, "0", LEFT );
}

void __fastcall TSetFilter::Button1Click(TObject *Sender)
{
  if ( pFilter!=NULL ) releaseFilter(gSide);
  gSide = UpDown1->Position;
  SetFilter->Refresh();
  showXY(SetFilter->Canvas, sx, sy);
  assignFilter((TWinControl*)SetFilter, gSide, gFilter);
}
//---------------------------------------------------------------------------

void __fastcall TSetFilter::Button2Click(TObject *Sender)
{
  releaseFilter(gSide);
  //showXY(SetFilter->Canvas, sx, sy);
  Refresh();
}
//---------------------------------------------------------------------------
void __fastcall TSetFilter::Button3Click(TObject *Sender)
{
  saveFilter(gFilter, gSide);
}
//---------------------------------------------------------------------------
void __fastcall TSetFilter::Button4Click(TObject *Sender)
{
  loadFilter(gFilter, gSide);
}
//---------------------------------------------------------------------------

void __fastcall TSetFilter::BitBtn1Click(TObject *Sender)
{
  if ( edAs->Text=="" ) return;
  assignFilter(StrToFloat(edAs->Text), gSide);
}
//---------------------------------------------------------------------------
void __fastcall TSetFilter::bbtnLoadFromFileClick(TObject *Sender)
{ TStringList *lst = new TStringList();

  if ( OpenDialog1->Execute() )
    { lst->LoadFromFile( OpenDialog1->FileName );
      for (int k=0; k<lst->Count; k++)
         { gFilter[k] = StrToFloat(lst->Strings[k]);
         }
      
      releaseFilter( gSide );
      gSide = (int)(sqrt(lst->Count));
      delete lst;
      UpDown1->Position = gSide;
      assignFilter((TWinControl*)SetFilter, gSide, gFilter);
    }
}
//---------------------------------------------------------------------------

void __fastcall TSetFilter::bbtnSaveToFileClick(TObject *Sender)
{ TStringList *lst;

  if ( SaveDialog1->Execute() )
    { if ( FileExists(SaveDialog1->FileName) )
        { if ( MessageDlg("ɮפwsbAnл\ɶܡH", mtConfirmation,
                          TMsgDlgButtons() << mbYes << mbNo, 0) != mrYes )
          return;
        }
      try { lst = new TStringList();
            for (int k=0; k<gSide*gSide; k++)
               lst->Add( pFilter[k]->Text );
            lst->SaveToFile( SaveDialog1->FileName );
          }
      __finally
        { delete lst;
        }
    }
}
//---------------------------------------------------------------------------

void __fastcall TSetFilter::bbtnSumClick(TObject *Sender)
{ float sum=0.0;

  for (int k=0; k<gSide*gSide; k++)
     { sum += gFilter[k];
     }
  ShowMessage( FloatToStr(sum) );
}
//---------------------------------------------------------------------------


void __fastcall TSetFilter::UpDown1Click(TObject *Sender,
      TUDBtnType Button)
{
  Edit1->Text = IntToStr(UpDown1->Position);        
}
//---------------------------------------------------------------------------

