//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "SmartFilter0.h"

#include "SetFilter0.h" // form for changing filter's value

#include <jpeg.hpp>    // needed for class TJPEGImage
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

extern float gFilter[]; // external data defined in SetFilter1.cpp
extern int   gSide;

const int RED   = 2;
const int GREEN = 1;
const int BLUE  = 0;
const int GRAY  = 4;

const float Deg2Rad = 3.141592653/180.;
const float Rad2Deg = 180./3.141592653;

// predefined filters
float sobelX[9] = {-1, 0, 1,
                   -2, 0, 2,
                   -1, 0, 1 };
float sobelY[9] = {-1,-2,-1,
                    0, 0, 0,
                    1, 2, 1 };
float prewittX[9] = {-1, 0, 1,
                     -1, 0, 1,
                     -1, 0, 1 };
float prewittY[9] = { 1, 1, 1,
                      0, 0, 0,
                     -1,-1,-1 };
float laplacian[9]= { 0, 1, 0,
                      1,-4, 1,
                      0, 1, 0 };
float marrHildreth[25] = { 0, 0,-1, 0, 0,
                           0,-1,-2,-1, 0,
                          -1,-2,16,-2,-1,
                           0,-1,-2,-1, 0,
                           0, 0,-1, 0, 0
                         };
float *userDefined;
float *filters[] = { sobelX, sobelY, prewittX, prewittY, laplacian, marrHildreth, userDefined };
int   sides[]   = { 3, 3, 3, 3, 3, 5, 5, 0 };
float gScale=1.0;  // for storing the scale factor of best fit.
int   cx[49],cy[49];

int next(int value, int len, int skip=1) // including both direction
{ value += skip;
  if ( skip<0 ) value += len;
  value %= len;
  return (value);
}

void setFilter(float *data, int len)
{ for (int k=0; k<len; k++)
     gFilter[k] = data[k];
}
void generateXY(int side)
{ int x,y,i,hf,index;

  if ( side>7 )
    { ShowMessage("Maximum size allowed here is 7.");
      return;
    }
  hf=side>>1;
  index=0;
  for (x=-hf; x<=hf; x++)
     { for (i=0; i<side; i++)
	  cx[index+i*side]=x;
       index++;
     }
  index=0;
  for (y=-hf; y<=hf; y++)
     { for (i=0; i<side; i++)
	  cy[index+i]=y;
       index+=side;
     }
}

void filterImage(TImage *image, int side, float *filter=NULL, float threshold=-1)
{ int width, height, area, r, c;
  unsigned char **imgLine, *imgResult;
  int hfSide, fx, fy;
  float value[3];
  Graphics::TBitmap *bmp= new Graphics::TBitmap;
  Graphics::TBitmap *bmpbuf= new Graphics::TBitmap;

  bmp->Width  = image->Picture->Bitmap->Width;
  bmp->Height = image->Picture->Bitmap->Height;
  bmp->PixelFormat = pf24bit;
  bmp->Canvas->Draw(0,0,image->Picture->Bitmap);

  bmpbuf->Width  = image->Picture->Bitmap->Width;
  bmpbuf->Height = image->Picture->Bitmap->Height;
  bmpbuf->PixelFormat = pf24bit;
  bmpbuf->Canvas->Draw(0,0,image->Picture->Bitmap);

  imgLine = new unsigned char *[side];
  width  = bmp->Width;
  height = bmp->Height;
  area = side*side;
  generateXY(side);
  hfSide = side/2;

  for (int r=0; r<bmp->Height; r++)
     { imgResult = (unsigned char *) bmp->ScanLine[r];
       int line=0;
       for (int k=-hfSide; k<=hfSide; k++)  // get the image lines where filter moves over.
          imgLine[line++] = (unsigned char*) bmpbuf->ScanLine[next(r,height,k)];
       int index=0;
       for (int c=0; c<bmp->Width; c++)
          { for (int p=0; p<3; p++) value[p] = 0.0;
            for (int k=0; k<area; k++)
               { fx = next(c,width,cx[k])*3;
                 fy = next(hfSide,side,cy[k]);
                 for (int p=0; p<3; p++)
                    value[p] += imgLine[fy][fx+p]*filter[k];
               }
            if ( threshold<0 )
              { for (int p=0; p<3; p++)
                   { if ( value[p]>255 ) value[p]=255;
                     else if ( value[p]<0 ) value[p]=0;
                     imgResult[index++]=(unsigned char) value[p];
                   }
              }
            else
              { for (int p=0; p<3; p++)
                   { if ( value[p]>255 ) value[p]=255;
                     else if ( value[p]<threshold ) value[p]=0;
                     imgResult[index++]=(unsigned char) value[p];
                   }
              }
          }
     }
  Form1->Canvas->Draw(0,0,bmp);
  delete bmp;
  delete bmpbuf;
  delete imgLine;
}

void __fastcall TForm1::Exit1Click(TObject *Sender)
{
  Close();        
}
//---------------------------------------------------------------------------
void __fastcall TForm1::Load1Click(TObject *Sender)
{ OpenDialog1->Filter = "*.jpg;*.jpeg;*.bmp|*.jpg;*.jpeg;*.bmp";

  if ( OpenDialog1->Execute() )
    { if ( ExtractFileExt(OpenDialog1->FileName).UpperCase()==".BMP" )
        { Image1->Picture->LoadFromFile(OpenDialog1->FileName);
          return;
        }
      Graphics::TBitmap *bmp= new Graphics::TBitmap;
      try { Image1->Picture->LoadFromFile(OpenDialog1->FileName);
            bmp->Height=Image1->Picture->Height;
            bmp->Width =Image1->Picture->Width;
            bmp->Canvas->Draw(0, 0, Image1->Picture->Graphic); // Graphic for JPEG format.
            Image1->Height=bmp->Height;
            Image1->Width =bmp->Width;
            Image1->Picture->Assign((Graphics::TBitmap*)bmp);  // transfering to BMP format
          }
      __finally
          { delete bmp;
          }
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::bbtnSetFilterClick(TObject *Sender)
{
  SetFilter->ShowModal();
}
//---------------------------------------------------------------------------


void __fastcall TForm1::ProcessClick(TObject *Sender)
{
  if ( Image1->Picture->Width*2 < Form1->ClientWidth )
    { Image1->Stretch = false;
      Image1->AutoSize = true;
      Image1->Left = Image1->Width+1;
    }
  else
    { Image1->Stretch = true;
      Image1->AutoSize = false;
      Image1->Width  = Image1->Picture->Width /3;
      Image1->Height = Image1->Picture->Height/3;
      Image1->Left = Form1->ClientWidth - Image1->Width -1;
    }
  //Image1->Refresh();
    
  try { Screen->Cursor = crHourGlass;
        if ( cboxFilter->ItemIndex==6 )  // User-definded case
          { if ( (chkThreshold->Checked) && (edThreshold->Text!="") )
              filterImage(Image1, gSide, gFilter, StrToInt(edThreshold->Text));
            else
               filterImage(Image1, gSide, gFilter);
          }
        else
          { if ( (chkThreshold->Checked) && (edThreshold->Text!="") )
              { filterImage(Image1, sides[cboxFilter->ItemIndex], filters[cboxFilter->ItemIndex],
                            StrToInt(edThreshold->Text));
              }
            else
              filterImage(Image1, sides[cboxFilter->ItemIndex], filters[cboxFilter->ItemIndex]);
          }
      }
   __finally
     { Screen->Cursor = crDefault;
     }
}
//---------------------------------------------------------------------------
int prevX, prevY;
bool drawing=false;

void __fastcall TForm1::grpFunctionMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  drawing = true;
  prevX = X;
  prevY = Y;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::grpFunctionMouseMove(TObject *Sender,
      TShiftState Shift, int X, int Y)
{
  if ( drawing )
    { int dx = X - prevX;
      int dy = Y - prevY;
      grpFunction->Left += dx;
      grpFunction->Top  += dy;
    }
}
//---------------------------------------------------------------------------

void __fastcall TForm1::grpFunctionMouseUp(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
  drawing = false;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::cboxFilterChange(TObject *Sender)
{
  if ( cboxFilter->ItemIndex<0 )
    cboxFilter->ItemIndex = 0;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormResize(TObject *Sender)
{
  grpFunction->Left = Form1->ClientWidth - grpFunction->Width -3;
  grpFunction->Top  = Form1->ClientHeight- grpFunction->Height-3;        
}
//---------------------------------------------------------------------------

